@@ -4,7 +4,12 @@ use strict;
 use warnings;
 use Alien::OpenMP;
 
-our $VERSION = q{0.0.6};
+our $VERSION = q{0.0.7};
+
+# This module is a wrapper around a ".h" file that is injected into Alien::OpenMP
+# via Inline:C's AUTO_INCLUDE feature. This header file constains C MACROs for reading
+# OpenMP relavent environmental variables via %ENV (set by OpenMP::Environment perhaps)
+# and using the standard OpenMP runtime functions to set them.
 
 sub Inline {
   my ($self, $lang) = @_;
@@ -204,7 +209,6 @@ void PerlOMP_2D_AoA_TO_2D_INT_ARRAY(SV *AoA, int numRows, int rowSize, int retAr
   }
 }
 
-
 /* TODO:
   * add unit tests for conversion functions
   * add some basic matrix operations (transpose for 2D, reverse for 1D)
@@ -222,158 +226,144 @@ __END__
 
 =head1 NAME
 
-OpenMP::Simple - Provides some DWIM C functions and MACROs, built on top of L<Alien::OpenMP>.
+OpenMP::Simple - Wrapper around C<Alien::OpenMP> that provides helpful C MACROs and
+runtime functions
 
 =head1 SYNOPSIS
 
-    use OpenMP::Simple;      # uses Alien::OpenMP, provides helpful C MACROs
-    use OpenMP::Environment; # setters/getters for important OpenMP environmental variables 
-    
-    use Inline (
-        C                 => 'DATA',
-        with              => qw/OpenMP::Simple/,
-    );
+  use strict;
+  use warnings;
+  use Test::More tests => 8;
+  
+  use OpenMP::Simple;
+  use OpenMP::Environment;
+  
+  use Inline (
+      C    => 'DATA',
+      with => qw/OpenMP::Simple/,
+  );
+  
+  my $env = OpenMP::Environment->new;
+  
+  note qq{Testing macro provided by OpenMP::Simple, 'PerlOMP_UPDATE_WITH_ENV__NUM_THREADS'};
 
-Note: while not C<experimental> per se, this module is still in a very fluid
-state. It is actively being used as the basis for real world codes. As
-more experience and intuition is gained by using it, this interface
-will only improve. Feedback is very valuable, so please reach out to L<<
-OODLER <oodler@cpan.org> >> or visit the I<Perl+OpenMP> Github repository,
-L<https://github.com/Perl-OpenMP>.
+  for my $num_threads ( 1 .. 8 ) {
+
+      my $current_value = $env->omp_num_threads($num_threads);
+
+      is _get_num_threads(), $num_threads, sprintf qq{The number of threads (%0d) spawned in the OpenMP runtime via OMP_NUM_THREADS, as expected}, $num_threads;
+
+  }
+  
+  __DATA__
+  __C__
+
+  int _get_num_threads() {
+    PerlOMP_UPDATE_WITH_ENV__NUM_THREADS // <~ e.g., MACRO to update number of threads per
+                                         //    OMP_NUM_THREADS + omp_set_num_threads OpenMP runtime call
+    int ret = 0;
+    #pragma omp parallel
+    {
+      #pragma omp single
+      ret = omp_get_num_threads();
+    }
+    return ret;
+  }
+  
+  __END__
+
+See the C<./t> directory for many more examples. It should be obvious,
+but C<Test::More> is not required; it's just for show and convenience here.
 
 =head1 DESCRIPTION
 
-C<OpenMP::Simple> is meant to work together with C<OpenMP::Environment>
+This module is a wrapper that provides a custom ".h" file, which is injected
+into L<Alien::OpenMP> via C<Inline:C>'s C<AUTO_INCLUDE> hook. This header
+file constains C MACROs for reading OpenMP relavent environmental variables
+via C<%ENV> (set preferably using L<OpenMP::Environment>) and by calling
+the standard OpenMP runtime functions to set them (e.g., C<OMP_NUM_THREADS>
+/ C<set_omp_num_threads>).
+
+C<OpenMP::Simple> is meant to work directly with C<OpenMP::Environment>
 in a way that provides the same runtime control experience that OpenMP's
 environmental variables provides.
 
-Contained here is essentially a set of macros and C<Inline::C> defined C<C>
-functions that make it easier for Perl programs to interface basic Perl
-structures more directly with C<Inline::C> functions for use with OpenMP It
-also containes macros that emulate the access of OpenMP programs to C<%ENV>
-at program start up, and through this module, during runtime.
+The most common use case is updating the number of OpenMP threads that are
+defined via C<OMP_NUM_THREADS>.
 
-Note: normal OpenMP programs do not re-sample C<%ENV> during runtime, but
-C<OpenMP::Simple> provides macros to do just that; so this module provides an
-additional level of control that so that C<OpenMP::Environment> may be used
-effectively during the running of a Perl program that contains C<Inline::C>
-functions with OpenMP inside of them.
+=head2 Experimental Parts
 
-As a reminder, the OpenMP standard has several aspects that work in concert
-to make shared memory programming (with NUMA/SMP threads) more accessible.
-These aspects may be summarized as the following:
-
-=over 4
-
-=item 1. a runtime and library of runtime callable functions
-
-that may be used to programatically affect execution and work distribution;
-here is where this module, L<OpenMP::Simple> makes a contribution to extending
-the promise of OpenMP to Perl programmers.
-
-=item 2. a set of environmental variables (i.e., C<%ENV>)
-
-that can be used to control the OpenMP runtime; this aspect is where the
-L<OpenMP::Environment> module comes into play.
-
-=item 3. work sharing constructs
-used to implicitly share memory and executions among a series of threads;
-these are implemented directly into the C<C> code, and is also where most
-of the hard work for anyone (including Perl programmers) exists. I.e.,
-making OpenMP more I<perlish>. It also may including in the future, using
-OpenMP I<inside> of the C<perl> interpreter itself; but this is the domain
-of core Perl maintainers and experts in language semantics.  No attempt has
-been made so far to address this large and very fruitful area of exploration.
-
-=back
-
-This module attempts to ease the transition for those more familiar with
-programming C with OpenMP than they are with Perl or using C<Inline::C>
-within their Perl programs. It build upon the configuration information that
-is provided for by C<Alien::OpenMP>, and appends to the C<AUTO_INCLUDE>
-literal lines of C code that defines useful macros and data conversion
-functions (Perl to C, C to Perl).
-
-In addition to helping to deal with getting data structures that are very
-common in the computational domains into and out of these C<Inline::C>'d
-routines that leverage I<OpenMP>, this module provides macros that are
-designed to provide behavior that is assumed to work when executing a binary
-that has been compiled with OpenMP support, such as the awareness of the
-current state of the C<OMP_NUM_THREADS> environmental variable.
+There is some attempt at helping to deal with getting data structures
+that are very common in the computational domains into and out of these
+C<Inline::C>'d routines that are parallized via OpenMP. We are currently
+investigating what is actually needed in this regard. It is possible that a lot
+of this is unnecessariy and it is likely that a large number of C<read-only>
+scenerios involving Perl internal data structures and OpenMP threads are
+actually I<thread-safe>. This does not address the potential knowledge gap
+for those who are more experienced with C<C> and OpenMP than they are with
+the Perl C API for accessing internal Perl data structures inside of C code.
 
 =head1 PROVIDED MACROS
 
 =head2 Updating Runtime with Environmental Variables
 
-When C<Inline::C> compiles and loads these shared libraries once, when the
-script is first loaded. At this time, the OpenMP runtime inspects C<%ENV>;
-but the environment is not automatically updated as whenever the runtime
-interfaces to the compiled library is used. The following macros offer
-a very concise and convenient way to update in the runtime based on the
-values of the respective environmental variables. They are designed with the
-use of C<OpenMP::Environment> in mind, which provides an OpenMP specific
-OOP interface to getting and setting these variables. Nearly all of them
-correspond to a way to C<set> them in the OpenMP runtime (provided by the
-compiler's implementation, e.g., C<omp.h>).
-
-All MACROS have at least 1 test. Please those in the Github repository to get
-an idea of how to use C<OpenMP::Simple>'s macros with C<OpenMP::Environment>.
+All MACROS have at least 1 test in the suite. Please look at these in the
+Github repository to get an idea of how to use C<OpenMP::Simple>'s macros
+with C<OpenMP::Environment>.
 
 =over 4
 
 =item C<PerlOMP_UPDATE_WITH_ENV__NUM_THREADS>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_NUM_THREADS}>, which is managed via C<<
-OpenMP::Environment->omp_num_threads[int numThreads]); >>.
+variable, C<$ENV{OMP_NUM_THREADS}>, which is managed via
+C<< OpenMP::Environment->omp_num_threads[int numThreads]); >>.
 
 =item C<PerlOMP_UPDATE_WITH_ENV__DEFAULT_DEVICE>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_DEFAULT_DEVICE}>, which is managed via C<<
-OpenMP::Environment->omp_default_device([int deviceNo]); >>.
+variable, C<$ENV{OMP_DEFAULT_DEVICE}>, which is managed
+via C<< OpenMP::Environment->omp_default_device([int deviceNo]); >>.
 
 =item C<PerlOMP_UPDATE_WITH_ENV__MAX_ACTIVE_LEVELS>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_MAX_ACTIVE_LEVELS}>, which is managed via C<<
-OpenMP::Environment->omp_max_active_levels([int maxLevel]); >>.
+variable, C<$ENV{OMP_MAX_ACTIVE_LEVELS}>, which is managed
+via C<< OpenMP::Environment->omp_max_active_levels([int maxLevel]); >>.
 
 =item C<PerlOMP_UPDATE_WITH_ENV__DYNAMIC>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_DYNAMIC}>, which is managed via C<<
-OpenMP::Environment->omp_dynamic(['true'|'false']); >>.
+variable, C<$ENV{OMP_DYNAMIC}>, which is managed
+via C<< OpenMP::Environment->omp_dynamic(['true'|'false']); >>.
 
 =item C<PerlOMP_UPDATE_WITH_ENV__NESTED>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_NESTED}>, which is managed via C<<
-OpenMP::Environment->omp_nested(['true'|'false']); >>.
+variable, C<$ENV{OMP_NESTED}>, which is managed
+via C<< OpenMP::Environment->omp_nested(['true'|'false']); >>.
 
 =item C<PerlOMP_UPDATE_WITH_ENV__SCHEDULE>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_SCHEDULE}>, which is managed via C<<
-OpenMP::Environment->omp_schedule(...); >>.
+variable, C<$ENV{OMP_SCHEDULE}>, which is managed
+via C<< OpenMP::Environment->omp_schedule(...); >>.
 
 Note: The schedule syntax is of the form I<schedule[;chunkSize]>.
 
 =item C<PerlOMP_UPDATE_WITH_ENV__TEAMS_THREAD_LIMIT>
 
 Updates the OpenMP runtime with the value of the environmental
-variable, C<$ENV{OMP_TEAMS_THREAD_LIMIT}>, which is managed via C<<
-OpenMP::Environment->omp_([int limit]); >>.
+variable, C<$ENV{OMP_TEAMS_THREAD_LIMIT}>, which is managed via C<< OpenMP::Environment->omp_([int limit]); >>.
 
-Note: Not supported until GCC 12.3.0
+Note: C<OMP_TEAMS_THREAD_LIMIT> is not supported until GCC 12.3.0
 
 =item C<PerlOMP_UPDATE_WITH_ENV__NUM_TEAMS>
 
 Updates the OpenMP runtime with the value of the environmental variable,
-C<$ENV{OMP_NUM_TEAMS}>, which is managed via C<< OpenMP::Environment->omp_([int
-num]); >>.
+C<$ENV{OMP_NUM_TEAMS}>, which is managed via C<< OpenMP::Environment->omp_([int num]); >>.
 
-Note: Not supported until GCC 12.3.0
+Note: C<OMP_NUM_TEAMS> is not supported until GCC 12.3.0
 
 =item C<PerlOMP_RET_ARRAY_REF_ret>
 
@@ -388,7 +378,7 @@ reference that's been populated via C<av_push>.
 B<Note>: Work is currently focused on finding the true limits of the Perl C
 API. It is likely that in a lot of cases, elements in Perl Arrays (AV) and Perl
 Hashes (HV) maybe accessed safely without first transferring the entire data
-structures into its C<pure C> equivalent.
+structures into its pure C<C> equivalent.
 
 =over 4
 
@@ -409,7 +399,8 @@ programs. It is designed to work together with L<OpenMP::Environment>.
 
 This module heavily favors the C<GOMP> implementation of the OpenMP
 specification within gcc. In fact, it has not been tested with any other
-implementations.
+implementations because L<Alien::OpenMP> doesn't support anything other
+than GCC at the time of this writing due to lack of anyone asking for it.
 
 L<https://gcc.gnu.org/onlinedocs/libgomp/index.html>
 
